#include <Arduino.h>
#define SHIFT_DATA 2
#define SHIFT_CLK 3
#define SHIFT_LATCH 4
#define D0 5
#define D7 12
#define WRITE_EN 13
#define OUTPUT_EN A0


void setAddress(int address) {
  shiftOut(SHIFT_DATA, SHIFT_CLK, MSBFIRST, address >> 8);
  shiftOut(SHIFT_DATA, SHIFT_CLK, MSBFIRST, address);

  digitalWrite(SHIFT_LATCH, LOW);
  digitalWrite(SHIFT_LATCH, HIGH);
  digitalWrite(SHIFT_LATCH, LOW);
}

void setDataBusOut()
{
  for(int i = D0; i <= D7; i++)
    pinMode(i, OUTPUT);
}

void setDataBusIn()
{
  for(int i = D0; i <= D7; i++)
    pinMode(i, INPUT);
}

void setByte(byte out)
{
  for(int i = D0; i <= D7; i++)
    digitalWrite(i, bitRead(out, i - D0));
}

byte readByte()                             //to read single byte present on the bus
{
  byte bajt = 0;
  for(int i = D0; i <= D7; i++)
    if(digitalRead(i))
      bitSet(bajt, i - D0);
  return bajt;
}

void writeByte(byte bajt, int adr)
{
  digitalWrite(OUTPUT_EN, HIGH);
  digitalWrite(WRITE_EN, HIGH);

  setAddress(adr);
  setByte(bajt);
  digitalWrite(WRITE_EN, LOW);
  digitalWrite(WRITE_EN, HIGH);
}

byte readData(int adr)                      //to read byte from the memory
{
  byte bajt = 0;
  setDataBusIn();

  digitalWrite(WRITE_EN, HIGH);
  digitalWrite(OUTPUT_EN, HIGH);

  setAddress(adr);
  digitalWrite(OUTPUT_EN, LOW);
  delayMicroseconds(1);
  bajt = readByte();
  digitalWrite(OUTPUT_EN, HIGH);

  return bajt;
}

void programData(byte bajt, int adr)
{
  setDataBusOut();
  writeByte(0xAA, 0x555);
  writeByte(0x55, 0x2AA);
  writeByte(0xA0, 0x555);
  writeByte(bajt, adr);
  delayMicroseconds(1);
}

void chipErase()
{
  setDataBusOut();
  writeByte(0xAA, 0x555);
  writeByte(0x55, 0x2AA);
  writeByte(0x80, 0x555);
  writeByte(0xAA, 0x555);
  writeByte(0x55, 0x2AA);
  writeByte(0x10, 0x555);
  delayMicroseconds(1);
}

//generated data
const byte data[11] = {B00100001, B11111111, B00000001, B11111001, B00111110, B00000001, B11010011, B00000000, B11000011, B00000100, B00000000};

void setup()
{
  pinMode(SHIFT_DATA, OUTPUT);
  pinMode(SHIFT_CLK, OUTPUT);
  pinMode(SHIFT_LATCH, OUTPUT);
  pinMode(WRITE_EN, OUTPUT);
  pinMode(OUTPUT_EN, OUTPUT);
  digitalWrite(OUTPUT_EN, HIGH);
  digitalWrite(WRITE_EN, HIGH);
  writeByte(0xf0, 0x0000); //reset
  Serial.begin(115200);
  delay(3000);

  Serial.println("programator FLASH v1.0 - gotowy."); 
}

void readSerial(byte bajt)
{
  switch(bajt)
  {
    case 'E':
      Serial.println("Czyszcze pamiec FLASH...");
      chipErase();
      delay(200);
      Serial.println("Wyczyszczono!");
      break;
    case 'W':
      Serial.println("Wgrywam przykladowy program...");
      for(int i = 0; i < sizeof(data); i++) programData(data[i], i);
      Serial.println("Wgrano pomyslnie!");
      break;
    case 'R':
      Serial.println("Odczytuje dane od adresu 0x0000...");
      for(int y = 0; y <= 249; y+=5)
      {
        for(int x = 0; x <= 4; x++)
        {
          Serial.print(readData(x+y), HEX);
          Serial.print("  ");
        }
        
        for(int x = 0; x <= 4; x++)
          Serial.print(char(readData(x+y)));

        Serial.println("");
      }
      break;
    case 'T':
      Serial.println("Resetuje pamiec...");
      setDataBusOut();
      writeByte(0xF0, 0x0000);
      setDataBusIn();
      break;
    default:
      Serial.println("Programator FLASH v1.0 - by Leoneq ;3");
      Serial.println("-------------------------------------------");
      Serial.println("E - Wyczysc pamiec");
      Serial.println("R - Odczytaj zawartosc pamieci");
      Serial.println("W - Wgraj przykladowy program");
      Serial.println("T - Zresetuj pamiec FLASH");
      break;
  }

}

void loop()
{
 if(Serial.available()) readSerial(Serial.read());
}